
from celery import task,chord,group,chain
from celery.signals import task_success
from celery.utils.log import get_task_logger

from django.conf import settings

from .models import *
from .managers import BulkManager,chunks

logger = get_task_logger(__name__)

def chunk_dict(d, n):
    allout = []
    temp = {}
    for k in d:
        if len(temp) == n:
            allout.append(temp)
            temp = {}
        temp[k] = d[k]
    allout.append(temp)
    return allout

@task()
def doReset():
    lemmas = loadLemmas()
    #lemma_job = group(addLemmas.si(x) for x in chunk_dict(lemmas,10000))
    wfs,spells,trans = loadWF()
    #wf_job = group(addWFS.si(x) for x in chunk_dict(wfs,10000))
    #spell_job = group(addSpells.si(x) for x in chunks(spells,10000))
    #trans_job = group(addTrans.si(x) for x in chunks(trans,10000))
    ajob = group([addLemmas.si(lemmas),addSpells.si(spells),addTrans.si(trans)])
    c2 = (deleteall.si() | loadCats.si() | ajob | addWFS.si(wfs) | addRelationships.si(wfs))
    res = c2()
    res.get()


@task()
def loadCats():
    cats = []
    with open(fetch_celex_resource('Category/cats.txt'),'r') as f:
        head = None
        for line in f:
            l = line.strip().split("\t")
            if head is None:
                head = l
                continue
            cats.append(SyntacticCategory(pk=int(l[0]),label=l[1],description=l[2],categoryType=l[3]))
    SyntacticCategory.objects.bulk_create(cats)

@task()
def deleteall():
    WordForm.objects.all().delete()
    Transcription.objects.all().delete()
    Orthography.objects.all().delete()
    Lemma.objects.all().delete()
    SyntacticCategory.objects.all().delete()

@task()
def addLemmas(lemmas):
    ls = []
    for key in lemmas:
        if int(lemmas[key]['CategoryNum']) > 12:
            continue
        cat = SyntacticCategory.objects.get(pk=int(lemmas[key]['CategoryNum']))
        ls.append(Lemma(pk=int(key),label=lemmas[key]['Word'],category=cat,frequency=int(lemmas[key]['Freq'])))
    Lemma.objects.bulk_create(ls)

def loadLemmas():
    lemmas = loadOrthLemmas()
    #lemmas = loadTransLemmas(lemmas)
    lemmas = addCategories(lemmas)
    return lemmas

def loadWF():
    wfs,spells = loadOrthWF()
    wfs,trans = loadTransWF(wfs)
    spells = [(i+1,v) for i,v in enumerate(spells)]
    trans = [(i+1,v) for i,v in enumerate(trans)]
    return wfs,spells,trans

@task()
def addRelationships(wfs):
    wf = WordForm.objects.all()
    pas = []
    sas = []
    for w in wf:
        for s in wfs[str(w.pk)]['Spellings']:
            spell = Orthography.objects.get(spelling = s[0])
            sas.append(SpelledAs(word_form=w,orthography=spell,frequency=s[1]))
        for t in wfs[str(w.pk)]['Transcriptions']:
            pron = Transcription.objects.get(transcription = t)
            sas.append(PronouncedAs(word_form=w,transcription=pron))
    SpelledAs.objects.bulk_create(sas)
    PronouncedAs.objects.bulk_create(pas)

@task()
def addWFS(wfs):
    wfss = []
    for key in wfs:
        lem = Lemma.objects.filter(pk=int(wfs[key]['IdLemma']))
        if len(lem) != 1:
            continue
        wfss.append(WordForm(pk=int(key),lemma = lem[0],frequency = int(wfs[key]['Freq'])))
    WordForm.objects.bulk_create(wfss)

@task()
def addSpells(spells):
    ss = [Orthography(pk=s[0],spelling = s[1]) for s in spells]
    Orthography.objects.bulk_create(ss)

@task()
def addTrans(trans):
    ts = [Transcription(pk=t[0],transcription = t[1]) for t in trans]
    Transcription.objects.bulk_create(ts)


def loadOrthLemmas():
    lemmas = {}
    with open(fetch_celex_resource('Orthography/celex-orthography-lemmas.txt'),'r') as f:
        head = None
        for line in f:
            l = line.strip().split("\\")
            if head is None:
                head = l
                continue
            main = l[:8]
            additional = l[8:]
            #spellings = [l[1]]
            nl = {'Word':l[1],'Freq':l[2]}
            #if len(additional) != 0:
            #    additional = chunks(additional,4)
            #    spellings.extend([re.sub(r'-(?!-)',r'',x[3]) for x in additional])
            #nl['Spellings'] = set(spellings)
            lemmas[l[0]] = nl
    return lemmas


def addCategories(lemmas):
    with open(fetch_celex_resource('Category/celex-syntax-lemmas.txt'),'r') as f:
        head = None
        for line in f:
            l = line.strip().split("\\")
            if head is None:
                head = l
                continue
            lemmas[l[0]]['CategoryNum'] = l[3]
    return lemmas

def loadOrthWF():
    ss = set([])
    wfs = {}
    with open(fetch_celex_resource('Orthography/celex-orthography-wordforms.txt'),'r') as f:
        head = None
        for line in f:
            l = line.strip().split("\\")
            if head is None:
                head = l
                continue
            main = l[:9]
            additional = l[9:]
            spellings = [(main[1],main[6])]
            nl = {'IdLemma':l[3],'Freq':l[2]}
            if len(additional) != 0:
                additional = chunks(additional,5)
                spellings.extend([(x[0],x[2]) for x in additional])
            nl['Spellings'] = set(spellings)
            ss.update([x[0] for x in nl['Spellings']])
            wfs[main[0]] = nl
    return wfs,ss

def loadTransWF(wfs):
    ts = set([])
    with open(fetch_celex_resource('Phonology/celex-phonology-wordforms.txt'),'r') as f:
        head = None
        for line in f:
            l = line.strip().split("\\")
            if head is None:
                head = l
                continue
            main = l[:9]
            additional = l[9:]
            trans = [main[6]]
            if len(additional) != 0:
                additional = chunks(additional,4)
                trans.extend([x[1] for x in additional])
            wfs[main[0]]['Transcriptions'] = set(trans)
            ts.update(trans)
    return wfs,ts

#def loadTransLemmas(lemmas):
#    with open(fetch_celex_resource('Phonology/celex-phonology-lemmas.txt'),'r') as f:
#        head = None
#        for line in f:
#            l = line.strip().split("\\")
#            if head is None:
#                head = l
#                continue
#            main = l[:8]
#            additional = l[8:]
#            trans = [main[5]]
#            if len(additional) != 0:
#                additional = chunks(additional,4)
#                trans.extend([x[1] for x in additional])
#            lemmas[l[0]]['Transcriptions'] = set(trans)
#    return lemmas
